import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_chessboard_js/chessboard/chessboard_notation.dart';
import 'package:flutter_chessboard_js/chessboard/utils.dart';
import 'board_square.dart';
import 'resizable_draggable.dart';
import 'chessboard_controller.dart';
import 'constants.dart';

/// Chessboard Widget, receives size and a chessboard controller
class ChessBoard extends StatefulWidget {
  ChessBoard({
    required this.initialSize,
    required this.controller,
    this.whiteSideTowardsUser = true,
    this.minSize = 400,
    this.boardType = BoardType.brown,
    this.notationPanelBackgroundColor = Colors.black,
    this.notationPanelWidth = 200,
    this.notationColor = Colors.white,
    this.showNotationPanel = true,
  }) : assert(
          initialSize >= minSize,
          'size property must be greater or equal to minSize',
        );

  final ChessboardController controller;
  final double minSize;
  final double initialSize;
  final BoardType boardType;
  final bool whiteSideTowardsUser;
  final Color notationPanelBackgroundColor;
  final double notationPanelWidth;
  final Color notationColor;
  final bool showNotationPanel;

  /// If the white side of the board is towards the user

  @override
  _ChessBoardState createState() => _ChessBoardState();
}

class _ChessBoardState extends State<ChessBoard> {
  late ChessboardController _controller;
  late double _size;
  late BoardType _boardType;
  late bool _whiteTowardUser;

  @override
  void initState() {
    _controller = widget.controller;
    _size = widget.initialSize;
    _boardType = widget.boardType;
    _whiteTowardUser = widget.whiteSideTowardsUser;
    super.initState();
  }

  @override
  void dispose() {
    widget.controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    /// Make sure the board size wont get messed up
    final screenSize = MediaQuery.of(context).size;
    final smallestSide = screenSize.width < screenSize.height
        ? screenSize.width
        : screenSize.height;
    if (_size > smallestSide) {
      _size = smallestSide;
    }
    if (_size < widget.minSize) _size = widget.minSize;

    return Padding(
      padding: const EdgeInsets.all(4.0),
      child: Container(
        decoration: BoxDecoration(
            borderRadius: BorderRadius.all(Radius.circular(10)),
            color: Colors.grey.withOpacity(0.4)),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Container(
              width: _size,
              height: _size,
              child: Stack(
                children: <Widget>[
                  Center(
                    child: Container(
                      width: _size * 0.95,
                      height: _size * 0.95,
                      child: getBoardImage(_boardType),
                    ),
                  ),
                  Center(
                    child: FittedBox(
                      fit: BoxFit.contain,
                      child: Container(
                        width: _size * 0.95,
                        height: _size * 0.95,
                        child: buildChessBoard(),
                      ),
                    ),
                  ),
                  Positioned(
                      right: 0,
                      bottom: 0,
                      child: ResizableDraggable(
                        size: _size / 20,
                        onDrag: (double offset) {
                          if (_size + offset >= widget.minSize &&
                              _size + offset <= smallestSide)
                            setState(() {
                              _size = _size + offset;
                            });
                        },
                      )),
                ],
              ),
            ),
            if ((screenSize.width > _size + 20 + widget.notationPanelWidth) &&
                screenSize.height > 300 &&
                widget.showNotationPanel)
              Padding(
                padding: const EdgeInsets.only(right: 8.0),
                child: ChessboardNotation(
                  flipBoard: flipBoard,
                  width: widget.notationPanelWidth,
                  height: _size * 0.95,
                  color: widget.notationPanelBackgroundColor,
                  controller: _controller,
                  notationTextColor: widget.notationColor,
                ),
              ),
          ],
        ),
      ),
    );
  }

  void flipBoard() {
    setState(() {
      _whiteTowardUser = !_whiteTowardUser;
    });
  }

  /// Builds the board, drawing each rank,
  /// on each rank each square
  Widget buildChessBoard() {
    return Column(
      children: _whiteTowardUser
          ? squareList.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row
                      .map((squareName) => BoardSquare(
                            key: ValueKey(squareName),
                            squareName: squareName,
                            size: _size,
                            controller: _controller,
                            moveCallback: (Map? move) {
                              if (move != null && mounted)
                                setState(() {
                                  _controller.gameLogic.move(move);
                                  _controller.refreshBoard();
                                });
                            },
                          ))
                      .toList(),
                ),
              );
            }).toList()
          : squareList.reversed.map((row) {
              return Expanded(
                flex: 1,
                child: Row(
                  children: row.reversed
                      .map((squareName) => BoardSquare(
                            key: ValueKey(squareName),
                            squareName: squareName,
                            size: _size,
                            moveCallback: (Map? move) {
                              if (move != null && mounted)
                                setState(() {
                                  _controller.gameLogic.move(move);
                                  _controller.refreshBoard();
                                });
                            },
                            controller: _controller,
                          ))
                      .toList(),
                ),
              );
            }).toList(),
    );
  }
}
